#include <algorithm>
#include <cstddef>
#include <cstdlib>
#include <chrono>
#include <cmath>
#include <iostream>
#include <random>
#include <stdint.h>
#define AVX_ALIGNMENT 32

using namespace std;
using namespace std::chrono;

typedef float floattype;

int main(int argc, char* argv[]) {

	uint32_t N = 0, A = 0, counter = 0;
	cin>>N;
	cin>>A;

	floattype *a, *r;
	posix_memalign((void**) &a, AVX_ALIGNMENT, N*sizeof(floattype));
	generate(a, a+N, [&]() { return (counter++)+0.5; });
	unsigned seed = std::chrono::system_clock::now().time_since_epoch().count();
	shuffle(a, a+N, default_random_engine(seed));

	posix_memalign((void**) &r, AVX_ALIGNMENT, N*sizeof(floattype));
	high_resolution_clock::time_point start = high_resolution_clock::now();

	#pragma omp parallel
	{
		floattype *b = nullptr;
		posix_memalign((void**) &b, AVX_ALIGNMENT, N*sizeof(floattype));
		#pragma omp for
		for (uint32_t j = 0; j < A; ++j)	{
			#pragma omp simd aligned(a,b:AVX_ALIGNMENT)
			for (uint32_t i = 0; i < N; ++i) {
				// note: suffix 'f' to function names for floats to enable vectorization
				b[i] = powf(sinf(a[i]), 2.0f);
				b[i] += powf(cosf(a[i]), 2.0f);
			}
		}

		#pragma omp critical
		copy(b, b+N, r);

		delete[] b;
	}

	double totalDuration = duration_cast<duration<double>>(high_resolution_clock::now()-start).count();

	if (argc > 1)	{
		for (uint32_t i = 0; i < N; ++i)
			cout<<" "<<r[i];
		cout<<endl;
	}
	cout<<"computational time: "<<totalDuration<<" s"<<endl;

	delete[] r;
	delete[] a;

	return 0;
}

